@@ -45,7 +45,9 @@ class Maevies(hildon.StackableWindow):
 
     def __init__(self):
         super(Maevies, self).__init__()
-        self.set_title('Maevies - 0.1')
+        self.set_title('%(name)s - %(version)s' %
+                {'name' : constants.APP_NAME,
+                 'version' : constants.APP_VERSION})
         self.connect('delete-event',
                      self.quit)
 
@@ -76,7 +78,7 @@ class Maevies(hildon.StackableWindow):
                                                 self.ACTION_THEATERS),
                             expand=True, fill=True)
         contents.pack_start(self._create_button('Favorites',
-                                                'Your saved searches',
+                                                'Your saved movies',
                                                 self.ACTION_FAVORITES),
                             expand=True, fill=True)
         contents.pack_start(self._create_button('Search',
@@ -90,6 +92,8 @@ class Maevies(hildon.StackableWindow):
         if action == self.ACTION_THEATERS:
             theaters_view = TheatersWindow()
             theaters_view.display_shows()
+        elif action == self.ACTION_FAVORITES:
+            favorites_window = FavoritesWindow()
         elif action == self.ACTION_SEARCH:
             search_dialog = SearchDialog(self)
             if search_dialog.run() == gtk.RESPONSE_ACCEPT:
@@ -181,15 +185,11 @@ class SearchDialog(gtk.Dialog):
     def get_search_category(self):
         return self._picker_button.get_active()
 
-class ResultsWindow(hildon.StackableWindow):
+class MovieListWindow(hildon.StackableWindow):
 
     def __init__(self):
-        super(ResultsWindow, self).__init__()
-        self.set_title('Search results')
-
+        super(MovieListWindow, self).__init__()
         self.add(self._create_contents())
-
-        moviemanager.response_received_cb = self._response_received_cb
         self.show_all()
 
     def _create_contents(self):
@@ -207,6 +207,40 @@ class ResultsWindow(hildon.StackableWindow):
         elif isinstance(movie, WatcMovie):
             WatcWindow(movie)
 
+class FavoritesWindow(MovieListWindow):
+
+    NO_FAVORITES = 'Mark a movie as favorite and it will appear here'
+
+    def __init__(self):
+        super(FavoritesWindow, self).__init__()
+        self.set_title('Favorites')
+        if moviemanager._favorites and self._movies_view:
+            self._movies_view.add_movies(moviemanager._favorites)
+
+    def _create_contents(self):
+        if moviemanager._favorites:
+            return super(FavoritesWindow, self)._create_contents()
+        no_items_label = gtk.Label(self.NO_FAVORITES)
+        no_items_label.set_justify(gtk.JUSTIFY_CENTER)
+        return no_items_label
+
+class ResultsWindow(MovieListWindow):
+
+    SEARCHING = 'Searching'
+    NO_RESULTS = 'No results found'
+    ERROR = 'Unable to perform search'
+
+    def __init__(self):
+        super(ResultsWindow, self).__init__()
+        self.set_title('Search results')
+        moviemanager.response_received_cb = self._response_received_cb
+
+    def _create_contents(self):
+        self._content = super(ResultsWindow, self)._create_contents()
+        self._background = gtk.Label(self.SEARCHING)
+        self._background.set_justify(gtk.JUSTIFY_CENTER)
+        return self._background
+
     def start_search(self, search_term, search_category):
         self._show_banner(self._get_search_message(search_term, search_category))
         hildon.hildon_gtk_window_set_progress_indicator(self, True)
@@ -214,11 +248,18 @@ class ResultsWindow(hildon.StackableWindow):
 
     def _response_received_cb(self, movies, error=None):
         hildon.hildon_gtk_window_set_progress_indicator(self, False)
-        if not error:
+        if not error and movies:
             self._movies_view.add_movies(movies)
+            self.remove(self._background)
+            self._content.show_all()
+            self.add(self._content)
         else:
-            self._show_banner(constants.DBUS_ERROR,
-                              constants.TIMEOUT_TIME_MILLIS * 5)
+            if error:
+                self._show_banner(constants.DBUS_ERROR,
+                                  constants.TIMEOUT_TIME_MILLIS * 5)
+                self._background.set_text(self.ERROR)
+            else:
+                self._background.set_text(self.NO_RESULTS)
 
     def _show_banner(self, message, timeout=constants.TIMEOUT_TIME_MILLIS):
         banner = hildon.hildon_banner_show_information_with_markup(self,
@@ -232,6 +273,7 @@ class ResultsWindow(hildon.StackableWindow):
                     'term' : search_term})
         return message
 
+
 class MoviesView(gtk.TreeView):
 
     def __init__(self):
@@ -357,6 +399,11 @@ class AboutDialog(gtk.Dialog):
 
 class MovieWindow(hildon.StackableWindow):
 
+    ACTION_TOGGLE_FAVORITE = 0
+
+    NON_FAVORITE_LABEL = 'Mark as favorite'
+    FAVORITE_LABEL = 'Unmark as favorite'
+
     def _fetch_movie_image(self, movie):
         image = gtk.Image()
         image.set_from_pixbuf(gtk.IconTheme().load_icon('general_video',
@@ -435,11 +482,40 @@ class MovieWindow(hildon.StackableWindow):
 
         return main_area
 
+    def _create_app_menu(self):
+        menu = hildon.AppMenu()
+
+        toggle_favorite = hildon.GtkButton(gtk.HILDON_SIZE_AUTO)
+        if self._movie.get_favorite():
+            toggle_favorite.set_label(self.FAVORITE_LABEL)
+        else:
+            toggle_favorite.set_label(self.NON_FAVORITE_LABEL)
+        toggle_favorite.connect('clicked',
+                                self._button_clicked,
+                                self.ACTION_TOGGLE_FAVORITE)
+
+        menu.append(toggle_favorite)
+
+        menu.show_all()
+
+        return menu
+
+    def _button_clicked(self, button, action):
+        if action == self.ACTION_TOGGLE_FAVORITE:
+            mark_favorite = button.get_label() == self.NON_FAVORITE_LABEL
+            if mark_favorite:
+                button.set_label(self.FAVORITE_LABEL)
+            else:
+                button.set_label(self.NON_FAVORITE_LABEL)
+            moviemanager.update_favorite(self._movie, mark_favorite)
+
     def __init__(self, movie):
+        self._movie = movie
         super(MovieWindow, self).__init__()
-        self.async_worker = AsyncWorker()
+        self.async_worker = AsyncWorker.get_instance()
         self.set_title('Movie info')
         self.add(self._create_contents(movie))
+        self.set_app_menu(self._create_app_menu())
         self.show_all()
 
 class WebkitWindow(hildon.StackableWindow):
