@@ -25,6 +25,8 @@ import dbus
 import gobject
 import gtk
 
+from lib import constants
+
 gobject.threads_init()
 glib.init_threads()
 
@@ -39,12 +41,13 @@ WATC = 1
 
 class MovieManager:
 
+    _favorites = []
+
     def __init__(self, response_received_cb=None):
         self.response_received_cb = response_received_cb
-        self._bus = None
         self._provider = None
-
         self._bus = dbus.SessionBus()
+        self.load_favorites()
 
     def _get_provider(self):
         if not self._provider:
@@ -77,17 +80,59 @@ class MovieManager:
 
         if interface == TMDB_MOVIE_INTERFACE:
             for path in object_paths:
-                movies.append(TmdbMovie(self._bus, path))
+                movies.append(TmdbMovie.from_dbus(self._bus, path))
         elif interface == WATC_MOVIE_INTERFACE:
             for path in object_paths:
-                movies.append(WatcMovie(self._bus, path))
+                movies.append(WatcMovie.from_dbus(self._bus, path))
 
         if self.response_received_cb:
             self.response_received_cb(movies, error)
 
+    def update_favorite(self, movie, favorite):
+        movie.set_favorite(favorite)
+        if favorite:
+            self._favorites.append(movie)
+        else:
+            self._favorites.remove(movie)
+        self.save_favorites()
+
+    def save_favorites(self):
+        try:
+            f = open(constants.MVS_FAVORITES_FILE, 'w')
+            try:
+                for favorite in self._favorites:
+                    f.write(repr(favorite) + '\n')
+            except:
+                f.close()
+        except:
+            pass
+
+    def load_favorites(self):
+        self._favorites = []
+        try:
+            f = open(constants.MVS_FAVORITES_FILE, 'r')
+            try:
+                for line in f:
+                    favorite = eval(line)
+                    self._favorites.append(favorite)
+            except:
+                f.close()
+        except:
+            pass
+
 class TmdbMovieImage:
-    def __init__(self, image_struct):
-        self._type, self._url, self._size, self._id = image_struct
+
+    def __init__(self, image_id, image_url, image_type, image_size):
+        self._id = image_id
+        self._url = image_url
+        self._type = image_type
+        self._size = image_size
+
+    @classmethod
+    def from_image_struct(cls, image_struct):
+        type, url, size, id = image_struct
+        return cls(image_id=id, image_url=url,
+                   image_type=type, image_size=size)
 
     def get_url(self):
         return self._url
@@ -107,15 +152,28 @@ class TmdbMovieImage:
             str += '\t%s\n' % field
         return str
 
+    def __repr__(self):
+        return ('TmdbMovieImage(image_id=%(id)r,'
+                ' image_url=%(url)r,'
+                ' image_type=%(type)r,'
+                ' image_size=%(size)r)' %
+                {'id' : self._id,
+                 'url' : self._url,
+                 'type' : self._type,
+                 'size' : self._size})
+
 class GenericMovie(object):
 
-    def __init__(self, bus):
-        self._bus = bus
-        pass
+    def __init__(self, dbus_interface=None):
+        self._interface = dbus_interface
 
-    def _create_interface(self, bus_name, bus_interface, object_path):
-        proxy = self._bus.get_object(bus_name,
-                                     object_path)
+    @classmethod
+    def from_dbus(cls, bus, object_path):
+        return cls()
+
+    @classmethod
+    def interface_from_dbus(cls, bus, bus_name, bus_interface, object_path):
+        proxy = bus.get_object(bus_name, object_path)
         interface = dbus.Interface(proxy,
                                    dbus_interface=bus_interface)
         return interface
@@ -141,13 +199,18 @@ class GenericMovie(object):
 
 class WatcMovie(GenericMovie):
 
-    def __init__(self, bus, object_path):
-        super(WatcMovie, self).__init__(bus)
-        self.interface = self._create_interface(WATC_MOVIE_BUS_NAME,
-                                                WATC_MOVIE_INTERFACE,
-                                                object_path)
+    def __init__(self, dbus_interface=None):
+        super(WatcMovie, self).__init__(dbus_interface)
         self.fields = ['Name', 'Year', 'Stingers']
 
+    @classmethod
+    def from_dbus(cls, bus, path):
+        interface = GenericMovie.interface_from_dbus(bus,
+                                                     WATC_MOVIE_BUS_NAME,
+                                                     WATC_MOVIE_INTERFACE,
+                                                     path)
+        return cls(dbus_interface=interface)
+
     def get_value(self, field):
         if field == 'Name':
             return self.get_title()
@@ -158,13 +221,13 @@ class WatcMovie(GenericMovie):
         super(WatcMovie, self).get_value()
 
     def get_title(self):
-        return self.interface.GetName()
+        return self._interface.GetName()
 
     def get_year(self):
-        return self.interface.GetYear()
+        return self._interface.GetYear()
 
     def get_stingers(self):
-        return self.interface.GetStingers()
+        return self._interface.GetStingers()
 
     def get_info(self):
         stingers = self.get_stingers()
@@ -184,43 +247,63 @@ class WatcMovie(GenericMovie):
 
 class TmdbMovie(GenericMovie):
 
-    def __init__(self, bus, object_path):
-        super(TmdbMovie, self).__init__(bus)
-        self.interface = self._create_interface(TMDB_MOVIE_BUS_NAME,
-                                                TMDB_MOVIE_INTERFACE,
-                                                object_path)
-        self.fields = ['Title', 'Release date', 'Rating', 'Popularity']
-        self._images = self._retrieve_images()
+    FIELD_TITLE = 'Title'
+    FIELD_RELEASE = 'Release date'
+    FIELD_RATING = 'Rating'
+    FIELD_POPULARITY = 'Popularity'
+
+    def __init__(self, dbus_interface=None,
+                 images={},
+                 title='',
+                 release_date='',
+                 rating=None,
+                 popularity=None,
+                 overview=None,
+                 favorite=False):
+        super(TmdbMovie, self).__init__(dbus_interface)
+        self.fields = [self.FIELD_TITLE,
+                       self.FIELD_RELEASE,
+                       self.FIELD_RATING,
+                       self.FIELD_POPULARITY]
+        self._data = {}
+        if dbus_interface:
+            self._images = self._retrieve_images()
+            self._data[self.FIELD_TITLE] = self._interface.GetTitle()
+            self._data[self.FIELD_RELEASE] = self._interface.GetReleased()
+            self._data[self.FIELD_RATING] = self._interface.GetRating()
+            self._data[self.FIELD_POPULARITY] = self._interface.GetPopularity()
+            self._overview = self._interface.GetOverview()
+        else:
+            self._images = images
+            self._data[self.FIELD_TITLE] = title
+            self._data[self.FIELD_RELEASE] = release_date
+            self._data[self.FIELD_RATING] = rating
+            self._data[self.FIELD_POPULARITY] = popularity
+            self._overview = overview
+        self._favorite = favorite
+
+    @classmethod
+    def from_dbus(cls, bus, path):
+        interface = cls.interface_from_dbus(bus,
+                                            TMDB_MOVIE_BUS_NAME,
+                                            TMDB_MOVIE_INTERFACE,
+                                            path)
+        return cls(dbus_interface=interface)
 
     def _retrieve_images(self):
-        images = { }
-        dbus_images = self.interface.GetImages()
+        images = {}
+        dbus_images = self._interface.GetImages()
         for dbus_image in dbus_images:
-            image = TmdbMovieImage(dbus_image)
+            image = TmdbMovieImage.from_image_struct(dbus_image)
             if not images.has_key(image.get_type()):
                 images[image.get_type()] = {}
             images[image.get_type()][image.get_size()] = image
         return images
 
     def get_value(self, field):
-        if field == 'Title':
-            return self.get_title()
-        elif field == 'Release date':
-            return self.get_released()
-        elif field == 'Rating':
-            return self.get_rating()
-        elif field == 'Popularity':
-            return self.get_popularity()
-        super(TmdbMovie, self).get_value()
-
-    def get_title(self):
-        return self.interface.GetTitle()
-
-    def get_popularity(self):
-        return self.interface.GetPopularity()
-
-    def get_rating(self):
-        return self.interface.GetRating()
+        if self._data.has_key(field):
+            return self._data[field]
+        super(TmdbMovie, self).get_value(field)
 
     def get_image(self, type, size):
         if self._images.has_key(type) and self._images[type].has_key(size):
@@ -228,17 +311,36 @@ class TmdbMovie(GenericMovie):
         else:
             return None
 
-    def get_released(self):
-        return self.interface.GetReleased()
-
     def get_info(self):
         return ('<b>%(title)s</b>\n<small><i>Popularity: </i>%(popularity)s'
                 '    <i>Rating: </i>%(rating)s    <i>Released: </i>'
                 '%(released)s</small>' %
-                {'title' : self.get_title(),
-                 'popularity' : self.get_popularity(),
-                 'rating' : self.get_rating(),
-                 'released' : self.get_released()})
+                {'title' : self._data[self.FIELD_TITLE],
+                 'popularity' : self._data[self.FIELD_POPULARITY],
+                 'rating' : self._data[self.FIELD_RATING],
+                 'released' : self._data[self.FIELD_RELEASE]})
 
     def get_overview(self):
-        return self.interface.GetOverview()
+        return self._overview
+
+    def get_favorite(self):
+        return self._favorite
+
+    def set_favorite(self, favorite):
+        self._favorite = favorite
+
+    def __repr__(self):
+        return ('TmdbMovie(images=%(images)r,'
+                ' title=%(title)r,'
+                ' release_date=%(release)r,'
+                ' rating=%(rating)r,'
+                ' popularity=%(popularity)r,'
+                ' overview=%(overview)r,'
+                ' favorite=%(favorite)r)' %
+                {'images' : self._images,
+                 'title' : self._data[self.FIELD_TITLE],
+                 'release' : self._data[self.FIELD_RELEASE],
+                 'rating' : self._data[self.FIELD_RATING],
+                 'popularity' : self._data[self.FIELD_POPULARITY],
+                 'overview' : self._overview,
+                 'favorite' : self._favorite})
